
/****************************************************************************
*  Read DS1820 Temperature sensor with One Wire Protocol
*  and send Temperature over CAN to master Node
*  EasyPIC6
*  PIC18F4585
* Master node dsPIC Pro 4 Board
* Slave node  EasyPIC 6 Board with CAN board(mcp2551)
****************************************************************************/
unsigned char Can_Init_Flags,Can_Send_Flags,Can_Rcv_Flags; // can flags
unsigned char Rx_Data_Len,Tx_Data_Len; // Rx /Tx data length in bytes
char RxTx_Data[8];                    // can rx/tx data buffer
char Msg_Rcvd;                       // reception flag
long Rx_ID,Tx_ID;
//------------------------------------------------------



//------------------------------------------------------
// function proto types
//------------------------------------------------------
void Init(void);
unsigned int ReadTempSensor(void);
void ReadCan(void);
void WriteCan(void);
//-------------------------------------------------------
// main code
//-------------------------------------------------------
void main()
{
    Init();
  //--- Main loop----------------------
  do{
     LATC.B0 ^=1;  // Blink Run led
     ReadCan();    // Read CAN and check if read temp command is received,
     // if command is received read temp and send to master
     if(RxTx_Data[0] == 'T'){WriteCan();RxTx_Data[0] = 0;}
     Delay_ms(80);
  }while (1);
}
//-------------------------------------------------------------------------
// Read DS1820 Temperature sensor with One Wire
unsigned int ReadTempSensor()
{
    unsigned int temp = 0;
 //--- Perform temperature reading
    Ow_Reset(&PORTA, 5);         // Onewire reset signal
    Ow_Write(&PORTA, 5,0xCC);    // Issue command SKIP_ROM
    Ow_Write(&PORTA, 5,0x44);    // Issue command CONVERT_T
    Delay_us(120);

    Ow_Reset(&PORTA, 5);
    Ow_Write(&PORTA, 5,0xCC);    // Issue command SKIP_ROM
    Ow_Write(&PORTA, 5,0xBE);    // Issue command READ_SCRATCHPAD

    temp =  Ow_Read(&PORTA,5);
    temp = (Ow_Read(&PORTA,5)<< 8) + temp;
    return temp;
}
//----------------------------------------------------------------------
// Read CAN and check for command to read Teperature
void ReadCan(void)
{
 Msg_Rcvd = CANRead(&Rx_ID,RxTx_Data,&Rx_Data_Len,&Can_Rcv_Flags);
 if(Msg_Rcvd !=0){
    Msg_Rcvd = 0; //clear Rx Flag
   switch (Rx_ID)
   {
    case 0x211:
    LATC.B1 ^=1; // Blink Rx Led ID0x211 is received
    break;
    case 0x213:
     asm nop;   // do nothing
    break;
   }//end switch
 }//end if
}//end ReadCan
//-----------------------------------------------------------------------
// Write the temperature to the master
void WriteCan(void)
{
 unsigned int TempRaw = 0;
 TempRaw = (ReadTempSensor()/0.01);//range 0-12500
 RxTx_Data[0] =(TempRaw &0xFF); //Raw Temp values in byte 0 and 1 Intel format
 RxTx_Data[1] =(TempRaw >>8);
 RxTx_Data[2]= 0;
 RxTx_Data[3]= 0;
 RxTx_Data[4]= 0;
 RxTx_Data[5]= 0;
 RxTx_Data[6]= 0;
 RxTx_Data[7]= 0;
 Tx_Data_Len = 8;
 Tx_ID = 0x210;
 CANWrite(Tx_ID,RxTx_Data,Tx_Data_Len,Can_Send_Flags);
 LATC.B2 ^=1; //Blink Tx Led
}
//-----------------------------------------------------------------------
// Initialize the application
void Init(void)
{
  ADCON1 = 0x0F;
  CMCON = 0x07;
  TRISC.B0 = 0; // RC0 output Run led
  TRISC.B1 = 0; // RC1 output Rx led
  TRISC.B2 = 0; // RC2 output Tx led
  LATC = 0;     // switch all leds off
 //---------------------------------------------------------------
 // Initialize CAN module
 //----------------------------------------------------------------
 Can_Init_Flags = 0;                    //
 Can_Send_Flags = 0;                    // clear flags
 Can_Rcv_Flags  = 0;                    //

 Can_Send_Flags = _CAN_TX_PRIORITY_0 &
                  _CAN_TX_STD_FRAME &
                  _CAN_TX_NO_RTR_FRAME;

 Can_Init_Flags = _CAN_CONFIG_SAMPLE_THRICE &
                  _CAN_CONFIG_PHSEG2_PRG_ON &
                  _CAN_CONFIG_STD_MSG &
                  _CAN_CONFIG_DBL_BUFFER_ON &
                  _CAN_CONFIG_VALID_STD_MSG &
                  _CAN_CONFIG_LINE_FILTER_OFF;

 CANInitialize(1,4,5,3,1,Can_Init_Flags);   //500Kb/sec @40Mhz
 CANSetOperationMode(_CAN_MODE_CONFIG,0xFF); // set CONFIGURATION mode
 CANSetMask(_CAN_MASK_B1,-1,_CAN_CONFIG_STD_MSG); // set all mask1 bits to ones
 CANSetMask(_CAN_MASK_B2,-1,_CAN_CONFIG_STD_MSG); // set all mask2 bits to ones
 CANSetFilter(_CAN_FILTER_B1_F1,0x211,_CAN_CONFIG_STD_MSG);
 CANSetOperationMode(_CAN_MODE_NORMAL,0xFF); // set NORMAL mode
}
//*******************************EOF****************************************