/****************************************************************************
* Master Node CAN Temperature OW Read
* Send command to EasyPIC6 to read DS1820 Temperature sensor EasyPIC6 Node
* send the temp value back over CAN and the master node display it on LCD
* dsPIC 30F6014A  dsPIC Pro 4 Dev board as Master Node
* EasyPIC6 as slave Node
*****************************************************************************/
#include "built_in.h"
//---------------------------------------------------------------------------
unsigned int Can_Init_Flags,
Can_Send_Flags, Can_Rcv_Flags;   // CAN flags
unsigned int Rx_Data_Len;        // received data length in bytes
unsigned int Tx_Data_Len;        // Tx data length in bytes
char Rx_Data[8];                 // Rx data buffer
char Tx_Data[8];                 // Tx data buffer
char Msg_Rcvd;                   // reception flag
unsigned long RxID;              // Rx IDs
unsigned long Tx_ID;             // Tx IDs
//---------------------------------------------------------------------------
// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB6_bit;
sbit LCD_D4 at LATD4_bit;
sbit LCD_D5 at LATD5_bit;
sbit LCD_D6 at LATD6_bit;
sbit LCD_D7 at LATD7_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB6_bit;
sbit LCD_D4_Direction at TRISD4_bit;
sbit LCD_D5_Direction at TRISD5_bit;
sbit LCD_D6_Direction at TRISD6_bit;
sbit LCD_D7_Direction at TRISD7_bit;
// End LCD module connections

//Set TEMP_RESOLUTION to the corresponding resolution of used DS18x20 sensor:
//18S20: 9  (default setting; can be 9,10,11,or 12)
//18B20: 12
const unsigned short TEMP_RESOLUTION = 9;
char *text = "000.0000";
unsigned char OldState,NewState ,TempAct;
unsigned temp;
//----------------------------------------------------------------------
struct ControlFlags
{
 unsigned getTemp :1;
 unsigned FanOn   :1;
 unsigned res :14;
}Flag;
//---------------------------------------------------------------------
// define the message filets to you ID's
//---------------------------------------------------------------------
#define F1  0x100
#define F2  0x101
#define F3  0x200
#define F4  0x210
#define F5  0x300
#define F6  0x310

#define TRUE  1
#define FALSE 0

//--------------------------------------------------------
// Proto Types
//--------------------------------------------------------
void Init(void);
void LcdInit(void);
void ReadCan(void);
void WriteCan(void);
void Display_Temperature(unsigned int temp2write);
//--------------------------------------------------------
// Timer 1 ISR
//--------------------------------------------------------
void Timer1Int() iv IVT_ADDR_T1INTERRUPT
{
  T1IF_bit = 0;        // Clear T1IF
  if(Flag.FanOn){Flag.getTemp = TRUE;}
  else{Flag.getTemp = FALSE;}
}
//--------------------------------------------------------
// main code
//--------------------------------------------------------
void main()
{
 //-----------------------------------------
 // Initialize Micro Controller
 //-----------------------------------------
   Init();
 //-----------------------------------------
 // Initialize LCD Display on startup
 //-----------------------------------------
   LcdInit();
 //-----------------------------------------
 // Ensure Application Start from know state
 //-----------------------------------------
    Flag.getTemp = FALSE;//Do not request Temp
    Flag.FanOn = FALSE;  // Not used
    OldState = NewState = 0; // put button scate to know state
    temp = 0;
 //--------------------------------------------------------
 // main code loop
 //--------------------------------------------------------
 do
 {
   //Check is user want to have the temperature read
   if (Button(&PORTB,0,300,0))
   {
    NewState = OldState ^=1;
    if(NewState==TRUE){Flag.FanOn = TRUE;LATC.B1 = TRUE;}
    else{Flag.FanOn = FALSE;LATC.B1 = FALSE;}
    OldState = NewState;
   }
   // Read CAN bus
    WriteCan();
    //Write CAN bus command to read Temperature to slave
    ReadCan();
    // execute when command is received
    if(Flag.FanOn){Display_Temperature(temp);}
    else {Display_Temperature(temp = 0);}
 }while(1);
}
//-------------------------------------------------------------------------
// Display Temperature on LCD
void Display_Temperature(unsigned int temp2write)
{
  const unsigned short RES_SHIFT = TEMP_RESOLUTION - 8;
  char temp_whole;
  unsigned int temp_fraction;

  // Check if temperature is negative
  if (temp2write & 0x8000) {
     text[0] = '-';
     temp2write = ~temp2write + 1;
     }

  // Extract temp_whole
   temp_whole = temp2write >> RES_SHIFT ;
   TempAct = temp_whole;
  // Convert temp_whole to characters
  if (temp_whole/100)
     text[0] = temp_whole/100  + 48;
  else
     text[0] = '0';

  text[1] = (temp_whole/10)%10 + 48;             // Extract tens digit
  text[2] =  temp_whole%10     + 48;             // Extract ones digit

  // Extract temp_fraction and convert it to unsigned int
  temp_fraction  = temp2write << (4-RES_SHIFT);
  temp_fraction &= 0x000F;
  temp_fraction *= 625;

  // Convert temp_fraction to characters
  text[4] =  temp_fraction/1000    + 48;         // Extract thousands digit
  text[5] = (temp_fraction/100)%10 + 48;         // Extract hundreds digit
  text[6] = (temp_fraction/10)%10  + 48;         // Extract tens digit
  text[7] =  temp_fraction%10      + 48;         // Extract ones digit

  // Print temperature on LCD
  Lcd_Out(2, 5, text);
}

//-------------------------------------------------------------------------
void ReadCan(void)
{
 // read the incomeing messages
 Msg_Rcvd = CAN2Read(&RxID , Rx_Data , &Rx_Data_Len, &Can_Rcv_Flags);
  if (Msg_Rcvd )
  {
    switch (RxID)
   {
    case  0x210 :      // mask out the ID's which you need
     temp = ((Rx_Data[1]*256) + Rx_Data[0]) * 0.01;
    break;
    case 0x200 :
     asm nop ;
    break;
   }// end switch case
  }
}
//------------------------------------------------------------------------
void WriteCan(void)
{
 if(Flag.getTemp)
 {
  Tx_Data[0] = 'T';
  Tx_Data[1] = TempAct;
  Tx_Data[2] = 0;
  Tx_Data[3] = 0;
  Tx_Data[4] = 0;
  Tx_Data[5] = 0;
  Tx_Data[6] = 0;
  Tx_Data[7] = 0;
  Tx_Data_Len = 8;
  Tx_ID = 0x211;
  CAN2Write(Tx_ID,Tx_Data, Tx_Data_Len,Can_Send_Flags);
 }
  Flag.getTemp = FALSE;
}
//--------------------------------------------------------------------------
void LcdInit(void)
{
 Lcd_Init();                                    // Initialize LCD
 Lcd_Cmd(_LCD_CLEAR);                           // Clear LCD
 Lcd_Cmd(_LCD_CURSOR_OFF);                      // Turn cursor off
 Lcd_Out(1, 1, " Temperature:   ");
 // Print degree character, 'C' for Centigrades
 Lcd_Chr(2,13,223);   // Print degree character, 'C' for Centigrades
 // Different LCD displays have different char code for degree
 // If you see greek alpha letter try typing 178 instead of 223
 Lcd_Chr(2,14,'C');
}
//--------------------------------------------------------------------------
void Init(void)
{
  ADPCFG = 0xFFFF;       // Configure AN pins as digital
  TRISBbits.TRISB0 = 1;  // RB0 as input fron switch on Fan Controller
  TRISCbits.TRISC1 = 0;  // Fan Led
 /*--------------Init Timer 1-------------------------------------------*/
  IPC0   = IPC0 | 0x1000;   // Interrupt priority level = 1
  T1IF_bit = 0;           // Clear T1IF
  T1IE_bit =1;          // Enable Timer1 interrupts
  T1CON = 0x8030;       // Timer1 ON, internal clock FCY, prescaler 1:256
 //-------------------Configure CAN 2 --------------------------------------
  Can_Init_Flags = 0;                              //
  Can_Send_Flags = 0;                              // clear flags
  Can_Rcv_Flags  = 0;                              //

  Can_Send_Flags = _CAN_TX_PRIORITY_0 &            // form value to be used
                   _CAN_TX_STD_FRAME &             // with CAN2Write
                   _CAN_TX_NO_RTR_FRAME;

  Can_Init_Flags = _CAN_CONFIG_SAMPLE_THRICE &     // form value to be used
                   _CAN_CONFIG_PHSEG2_PRG_ON &     // with CAN2Initialize
                   _CAN_CONFIG_STD_MSG &
                   _CAN_CONFIG_DBL_BUFFER_ON &
                   _CAN_CONFIG_MATCH_MSG_TYPE &
                   _CAN_CONFIG_LINE_FILTER_OFF;

  CAN2Initialize(1,2,5,3,1,Can_Init_Flags) ;  //Set baud rate 500Kb/sec
 // CAN1Initialize(1,10,3,3,1,Can_Init_Flags) ; //125Kb/sec @80Mhz
//  CAN1Initialize(1,5,4,2,1,Can_Init_Flags) ;  //250Kb/sec @80Mhz

  C2CTRL.CANCKS = 1;  // Set CAN clock to Fcy
  CAN2SetOperationMode(_CAN_MODE_CONFIG,0xFF); // set CONFIGURATION mode
  CAN2SetMask(_CAN_MASK_B1,-1,_CAN_CONFIG_MATCH_MSG_TYPE & _CAN_CONFIG_STD_MSG);
  CAN2SetMask(_CAN_MASK_B2,-1,_CAN_CONFIG_MATCH_MSG_TYPE & _CAN_CONFIG_STD_MSG);
  CAN2SetFilter(_CAN_FILTER_B1_F1,F1,_CAN_CONFIG_STD_MSG);
  CAN2SetFilter(_CAN_FILTER_B1_F2,F2,_CAN_CONFIG_STD_MSG);
  CAN2SetFilter(_CAN_FILTER_B2_F1,F3,_CAN_CONFIG_STD_MSG);
  CAN2SetFilter(_CAN_FILTER_B2_F2,F4,_CAN_CONFIG_STD_MSG);
  CAN2SetFilter(_CAN_FILTER_B2_F3,F5,_CAN_CONFIG_STD_MSG);
  CAN2SetFilter(_CAN_FILTER_B2_F4,F6,_CAN_CONFIG_STD_MSG);
  CAN2SetOperationMode(_CAN_MODE_NORMAL,0xFF);//set normal mode
  Msg_Rcvd = 0;

}
//*******************************EOF****************************************